#include "data.h";
#include "pad.h";
#include "ressource.h";

void waitForVBlank(void) {
	byte Status;
	do {
		Status = *(byte*)0x4210;
	} while (!(Status & 0x80));
}

void VRAMLoad(word src, word vramDst, word size) {
	// set address in VRam for read or write ($2116) + block size transfer ($2115)
	*(byte*)0x2115 = 0x80;	
	*(word*)0x2116 = vramDst;

	*(word*)0x4300 = 0x1801;	// set DMA control register (1 word inc) and destination ($21xx xx -> 0x18)
	*(word*)0x4302 = src;		// DMA channel x source address offset 
					// (low $4302 and high $4303 optimisation)
	*(byte*)0x4304 = 0x00; 		// DMA channel x source address bank
	*(word*)0x4305 = size;		// DMA channel x transfer size 
					// (low $4305 and high $4306 optimisation)

	// Turn on DMA transfer for this channel
	waitForVBlank();
	*(byte*)0x2100 = 0x80;
	*(byte*)0x420b = 0x01;
	*(byte*)0x2100 = 0x00;
}

void CGRAMLoad(word src, byte cgramDst, word size) {
	
	// set address in VRam for read or write + block size
	*(byte*)0x2121 = cgramDst;

	*(word*)0x4300 = 0x2200;	// set DMA control register (1 byte inc) and destination ($21xx xx -> 022)
	*(word*)0x4302 = src;		// DMA channel x source address offset
					// (low $4302 and high $4303 optimisation)
	*(byte*)0x4304 = 0x00; 		// DMA channel x source address bank
	*(word*)0x4305 = size;		// DMA channel x transfer size
					// (low $4305 and high $4306 optimisation)

	// Turn on DMA transfer for this channel
	waitForVBlank();
	*(byte*)0x2100 = 0x80;
	*(byte*)0x420b = 0x01;
	*(byte*)0x2100 = 0x00;
}

void preInit(void) {
	// For testing purpose ... 
	// Insert code here to be executed before register init
}

void main(void) {

	word padValue1;
	padStatus *pad1;
	byte HScroll = 0;

	// Pointer hacker for typecasting padStatus
	pad1 = (padStatus *) &padValue1;

	*(byte*) 0x2107 = 0x10; // Screen map data @ VRAM location $1000
	*(byte*) 0x210b = 0x02; // Plane 0 Tile graphics @ $2000

	VRAMLoad((word) title_pic, 0x2000, 0x1BE0);
	VRAMLoad((word) title_map, 0x1000, 0x0800);
	CGRAMLoad((word) title_pal, (byte) 0x00, (word) 256);
	
	*(byte*) 0x2105 = 0x01;	// MODE 1 value

	*(byte*) 0x212c = 0x01; // Plane 0 (bit one) enable register
	*(byte*) 0x212d = 0x00;	// All subPlane disable

	*(byte*) 0x2100 = 0x0f; // enable background
	
	// Loop forever
	while(1) {
		waitForVBlank();

		// Read pad1 value
		padValue1 = readPad((byte) 0);
	
		if(pad1->left) {
			HScroll++;
		} else if(pad1->right) {
			HScroll--;
		}

		// Plane 0 scroll register
		*(byte*) 0x210d = HScroll;
	}
}

void IRQHandler(void) {
}
